class Auth {

    /**
     * 
     * @param {String} username login email
     * @returns result object array
     */

    async validateEmail( username ) {
        
        let query = `SELECT * FROM proll_employee pe WHERE pe.loginname LIKE ?`;

        let db = global.db;

        let results = await db.runPoolQuery(query, [username]);

        return results;

    }
  /**
   * 
   * @param {Number} id 
   * @returns 
   */

    async findEmployee(id) {

      let query = `SELECT * FROM proll_employee pe WHERE pe.id = ?`;

      let db = global.db;

      let results = await db.runPoolQuery(query, [id]);

      return results;
    }
    /**
     * 
     * @param {String} username login email
     * @param {String} password md5 encoded password
     * @returns result object array
     */

    async validatePassword( username, password ) {

        let query = `SELECT * FROM proll_employee pe WHERE pe.loginname LIKE ?
                    AND pe.e_pass_word LIKE ?`;

        let db = global.db;

        let results = await db.runPoolQuery(query, [username, password]);

        return results;

    }

    /**
     * 
     * @param {Number} userLoginId login user id
     * @returns result object array
     */

    async accessUser(userLoginId) {

        let query = `SELECT pe.loginname AS username, pe.e_pass_word AS secret, 
        mll.destination,
        (
            SELECT
            CASE 
                WHEN (mll.portal = 'hr' AND mll.role = 'hr') THEN mll.portal
                WHEN (mll.portal = 'hr' AND mll.role = 'lm') THEN mll.role
                WHEN (mll.portal = 'hr' AND mll.role = 'em') THEN mll.role
                WHEN (mll.portal = 'lm' AND mll.role = 'lm') THEN mll.portal
                WHEN (mll.portal = 'lm' AND mll.role = 'hr') THEN mll.role
                WHEN (mll.portal = 'lm' AND mll.role = 'em') THEN mll.role
                WHEN (mll.portal = 'em' AND mll.role = 'em') THEN mll.portal
                WHEN (mll.portal = 'em' AND mll.role = 'lm') THEN mll.role
                WHEN (mll.portal = 'em' AND mll.role = 'hr') THEN mll.role
                WHEN (mll.portal = 'ceo' AND mll.role = 'ceo') THEN mll.portal
                WHEN (mll.portal = 'ceo' AND mll.role = 'hr') THEN mll.portal
                WHEN (mll.portal = 'ceo' AND mll.role = 'lm') THEN mll.role
                WHEN (mll.portal = 'ceo' AND mll.role = 'em') THEN mll.role
                WHEN (mll.portal = 'hr' AND mll.role = 'ceo') THEN mll.role
                WHEN (mll.portal = 'lm' AND mll.role = 'ceo') THEN mll.role
                WHEN (mll.portal = 'em' AND mll.role = 'ceo') THEN mll.role
                WHEN (mll.portal = 'cr' AND mll.role = 'cr') THEN mll.portal
                WHEN (mll.portal = 'cr' AND mll.role = 'hr') THEN mll.portal
                WHEN (mll.portal = 'cr' AND mll.role = 'lm') THEN mll.role
                WHEN (mll.portal = 'cr' AND mll.role = 'em') THEN mll.role
                WHEN (mll.portal = 'hr' AND mll.role = 'cr') THEN mll.role
                WHEN (mll.portal = 'lm' AND mll.role = 'cr') THEN mll.role
                WHEN (mll.portal = 'em' AND mll.role = 'cr') THEN mll.role
            ELSE
            mll.role
            END
            ) portal,
        mll.id FROM multi_login ml
        INNER JOIN proll_employee pe ON pe.id = ml.employee_id
        INNER JOIN multi_login_log mll ON mll.multi_login_id = ml.id
        WHERE ml.id = '?' AND mll.des_portal LIKE 'angular_ppi_people' 
        ORDER BY mll.id DESC LIMIT 0, 1`;

        let db = global.db;

        let results = await db.runPoolQuery(query, [userLoginId]);

        return results;
    }

    /**
     * 
     * @param {String} username login email of user
     * @param {String} password md5 encrypted password
     * @returns result object array
     */

    async checkUser(username, password) {

        let checkUser = await this.validateEmail(username);

        let checkPassword = await this.validatePassword(username, password);

        if(!checkUser || checkUser.length == 0) {
            return {
                "response": false,
                "err": "Invalid Username Provided",
            };
        }
        else if(!checkPassword || checkPassword.length == 0) {
            return {
                "response": false,
                "err": "Invalid Password Provided",
            };
        }
        else {
            return {
                "response": true,
                "email": checkPassword[0].loginname,
                "code": checkPassword[0].pass_word,
                "id": checkPassword[0].id,
                "empcode": checkPassword[0].empcode,
                "emp_name": checkPassword[0].name,
                "client_id": checkPassword[0].cid,
            }

        }
        

    }
    
    /**
     * 
     * @param {Number} employeeId employee id
     * @returns result object array
     */

    async checkSeamlessLogin( employeeId ) {

        if( Number.isInteger(employeeId) ) {

            let query = "SELECT * FROM `multi_login` WHERE employee_id = ?";

            let db = global.db;

            let results = await db.runPoolQuery(query, [employeeId]);

            return results;

        }
        else {
            return false;
        }

    }

    /**
     * 
     * @param {String} source string of source
     * @param {String} destination string of destination
     * @param {String} portal string of portal
     * @param {String} remoteAddr remote http address
     * @param {String} agent request agent
     * @param {Number} employeeId id of user
     * @param {Number} isLogin boolean check 1 for true, 0 for false
     * @param {DateTime} loginTime current date and time
     * @param {JSON} extraParams extra request parameters in json
     * @returns result object array
     */

    async insertUserData(source, destination, portal, remoteAddr, agent, 
                        employeeId, isLogin, loginTime, extraParams) {

        let loginData  = await this.insertMultiLoginData(employeeId, isLogin, loginTime);
        let insertLoginLog = await this.insertMultiLoginLogData(employeeId, source, 
                                                                destination, remoteAddr, agent, portal,
                                                                extraParams);
        let loginId = await this.checkSeamlessLogin(employeeId);

        if(loginData.affectedRows > 0 && insertLoginLog.affectedRows > 0) {
        
          return {
            'status' : '200',
            'loginId' : loginId[0].id
          };
  
        }
        else {
          
          return {
            'status' : '500',
            'loginId' : false
          };
        }
        
    }

    /**
     * 
     * @param {String} source string of source
     * @param {String} destination string of destination
     * @param {String} portal string of portal
     * @param {String} remoteAddr remote http address
     * @param {String} agent request agent
     * @param {Number} employeeId id of user
     * @param {Number} isLogin boolean check 1 for true, 0 for false
     * @param {DateTime} loginTime current date and time
     * @param {JSON} extraParams extra request parameters in json
     * @returns result object array
     */

    async updateUserData(source, destination, portal, remoteAddr, agent,
                  employeeId, isLogin, loginTime, extraParams) {

      let loginData  = await this.updateMultiLoginData(employeeId, isLogin, loginTime);
      let insertLoginLog = await this.insertMultiLoginLogData(employeeId, source, 
                                                              destination, remoteAddr, agent, portal,
                                                              extraParams);
      let loginId = await this.checkSeamlessLogin(employeeId);
      
      if(loginData.affectedRows > 0 && insertLoginLog.affectedRows > 0) {
        
        return {
          'status' : '200',
          'loginId' : loginId[0].id
        };

      }
      else {
        
        return {
          'status' : '500',
          'loginId' : false
        };

      }
      
    }

    /**
     * 
     * @param {Number} employeeId id of user
     * @param {Number} isLogin boolean check 1 for true, 0 for false
     * @param {DateTime} loginTime current date and time
     * @returns insert query results
     */


    async insertMultiLoginData(employeeId, isLogin, loginTime) {

      let query = "INSERT INTO `multi_login` (`employee_id`, `is_login`, `login_time`) \n\
                  VALUES (?, ?, ?)";

        let db = global.db;

        let results = await db.runPoolQuery(query, [employeeId, isLogin, loginTime]);

        return results;

    }

    /**
     * 
     * @param {Number} employeeId id of user
     * @param {Number} isLogin boolean check 1 for true, 0 for false
     * @param {DateTime} loginTime current date and time
     * @returns update query results
     */

    async updateMultiLoginData(employeeId, isLogin, loginTime) {

        let query = "UPDATE `multi_login` \n\
                  SET `is_login` = ?, `login_time`= ? \n\
                  WHERE `employee_id` = ?";
      
        let db = global.db;

        let results = await db.runPoolQuery(query, [isLogin, loginTime, employeeId]);

        return results;

    }

    /**
     * 
     * @param {String} source string of source
     * @param {String} destination string of destination
     * @param {String} portal string of portal
     * @param {String} addr remote http address
     * @param {String} agent request agent
     * @param {Number} employeeId id of user
     * @param {JSON} extraParams extra request parameters in json
     * @returns insert query results
     */

    async insertMultiLoginLogData(employeeId, source, destination, addr, agent, portal, extraParams) {

      let loginUser = await this.checkSeamlessLogin(employeeId);

      let params = '';

      if(extraParams == '') {

        params = extraParams;

      }
      else {

        params = decodeURIComponent(extraParams);

      }

      if(loginUser.length > 0) {

        let query = "INSERT INTO `multi_login_log` \n\
                        (`multi_login_id`, `source`, `destination`, `client_ip`, `client_details`, \n\
                        `portal`, `is_login`, `src_portal`, `des_portal`, `params`)\n\
                        VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";

        let paramsArray = [

            loginUser[0].id,
            source,
            destination,
            addr,
            agent,
            portal,
            '1',
            'angular_ppi_people',
            'php_ppi_people',
            params

        ];

        let db = global.db;

        let results = await db.runPoolQuery(query, paramsArray);

        return results;

      }
      else {

        return false;

      }

    }

}

exports.Auth = Auth;