const generalMethods = require("../interface/GeneralMethodsInterface");

const GeneralApisData = require("../model/GeneralApisData").GeneralApisData;

class GeneralApisController {

  generalApisData = new GeneralApisData();

  getEmployeeInfoByEmpCodeEmail() {

    return async (req, res) => {

      try {
        let emp_code = req.query.emp_code;
        let email = req.query.email;    
    
        let result = await this.generalApisData.getEmployeeInfoByCodeOrEmail(
          emp_code,
          email
        );
    
        res.status(200).json({
          status: true,
          statusCode: 200,
          message: "success",
          payload: result,
        });
      } catch (error) {
        console.log(error);
        res.status(500).json({
          status: false,
          statusCode: 500,
          message: "internal failure",
          payload: null,
        });
      }


    }

  }

  getEmployeeContactInfoByEmpCode() {

    return async (req, res) => {

      try {
        let emp_code = req.query.emp_code;    
    
        let result = await this.generalApisData.getEmployeeContactInfoByCode(
          emp_code
        );
    
        res.status(200).json({
          status: true,
          statusCode: 200,
          message: "success",
          payload: result,
        });
      } catch (error) {
        console.log(error);
        res.status(500).json({
          status: false,
          statusCode: 500,
          message: "internal failure",
          payload: null,
        });
      }
      
    }

  }

  getDirectEmployeeListByLm() {

    return async (req, res) => {

      try {
        let emp_code = req.query.emp_code;    
    
        let result = await this.generalApisData.getDirectEmployeeListByLm(
          emp_code
        );
    
        res.status(200).json({
          status: true,
          statusCode: 200,
          message: "success",
          payload: result,
        });
      } catch (error) {
        console.log(error);
        res.status(500).json({
          status: false,
          statusCode: 500,
          message: "internal failure",
          payload: null,
        });
      }
      
    }

  }

  getEmployeeAllLmListByEmpCode() {

    return async (req, res) => {

      try {
        let emp_code = req.query.emp_code;    
    
        this.generalApisData
          .getEmployeeAllLmListByEmpCode(emp_code)
          .then((result) => {
            //console.log(result);
            res.status(200).json({
              status: true,
              statusCode: 200,
              message: "success",
              payload: result,
            });
          })
          .catch((error) => {
            console.log(error);
            res.status(500).json({
              status: false,
              statusCode: 500,
              message: "internal failure",
              payload: null,
            });
          });
      } catch (error) {
        console.log(error);
      }
      
    }

  }

  getAllEmployeesListByLmCode() {

    return async (req, res) => {

      try {
        let emp_code = req.query.emp_code;
        let lm_level = req.query.lm_level;    
    
        this.generalApisData
          .getAllEmployeesListByLmCode(emp_code)
          .then((result) => {
            //console.log(result);
            res.status(200).json({
              status: true,
              statusCode: 200,
              message: "success",
              payload: result,
            });
          })
          .catch((error) => {
            console.log(error);
            res.status(500).json({
              status: false,
              statusCode: 500,
              message: "internal failure",
              payload: null,
            });
          });
      } catch (error) {
        console.log(error);
      }
      
    }

  }

  getAllEmployeesListByHrCode() {

    return async (req, res) => {

      try {
        let emp_code = req.query.emp_code;    
    
        this.generalApisData
          .getAllEmployeesListByHrCode(emp_code)
          .then((result) => {
            //console.log(result);
            res.status(200).json({
              status: true,
              statusCode: 200,
              message: "success",
              payload: result,
            });
          })
          .catch((error) => {
            console.log(error);
            res.status(500).json({
              status: false,
              statusCode: 500,
              message: "internal failure",
              payload: null,
            });
          });
      } catch (error) {
        console.log(error);
      }
      
    }

  }

  getEmployeeInfoByGeographicalInfo() {

    return async (req, res) => {

      try {
        let emp_code = req.query.emp_code;
        // let group_level = req.body.group_level;
        // let group_level = req.body.group_level;
        // let group_level = req.body.group_level;
        let country = req.body.country;
        let province = req.body.province;
        let city = req.body.city;
        let branch = req.body.branch;
        let company = req.body.company;
        let band = req.body.band;
        let department = req.body.department;
        
    
        this.generalApisData
          .getEmployeeInfoByGeographicalInfo(
            country,
            province,
            city,
            branch,
            company,
            band,
            department
          )
          .then((result) => {
            //console.log(result);
    
            res.status(200).json({
              status: true,
              statusCode: 200,
              message: "success",
              payload: result,
            });
          })
          .catch((error) => {
            console.log(error);
            res.status(500).json({
              status: false,
              statusCode: 500,
              message: "internal failure",
              payload: null,
            });
          });
      } catch (error) {
        console.log(error);
      }
      
    }

  }

  getEmployeeVerificationByLm() {

    return async (req, res) => {

      try {
        let emp_code = req.query.emp_code;
        let lm_code = req.query.lm_code;
        
    
        let result = await this.generalApisData.getEmployeeVerificationByLm(
          emp_code,
          lm_code
        );
    
        //console.log(result);
    
        if (Object.keys(result).length > 0) {
          res.status(200).json({
            status: true,
            statusCode: 200,
            message: "success",
            payload: result,
          });
        }
      } catch (error) {
        console.log(error);
        res.status(500).json({
          status: false,
          statusCode: 500,
          message: "internal failure",
          payload: null,
        });
      }
      
    }

  }

  getEmployeeVerificationByHr() {

    return async (req, res) => {

      try {
        let emp_code = req.query.emp_code;
        let hr_code = req.query.hr_code;
        
    
        let result = await this.generalApisData.getEmployeeVerificationByHr(
          emp_code,
          hr_code
        );
    
        //console.log(result);
    
        if (Object.keys(result).length > 0) {
          res.status(200).json({
            status: true,
            statusCode: 200,
            message: "success",
            payload: result,
          });
        }
      } catch (error) {
        console.log(error);
        res.status(500).json({
          status: false,
          statusCode: 500,
          message: "internal failure",
          payload: null,
        });
      }
      
    }

  }

  dummy() {

    return async (req, res) => {

      let result = await generalMethods.getEmployeeAllLmListByEmpCode(175695);
      console.log(result);
      res.status(200).json({
        result,
      });
      
    }

  }

}

exports.GeneralApisController = GeneralApisController;