const mongoose = require('mongoose');

const menusCacheSchema = mongoose.Schema({
    e_number: Number,
    link: String,
  
    levelOne: Number,
    levelTwo: Number,
    levelThree: Number,
});
  
  // Menus Schema for all portals, this will be static and will be updated when any menu portal is updated
const menus = mongoose.Schema({
    id: Number,
    menu_title: String,
    parent_id: Number,
    menu_order: Number,
    menu_type_id: Number,
    level: Number,
    client_id: Number,
    created_by: Number,
    created_at: Date,
    updated_at: String,
    icon: String,
    img: String,
    page_menu_title: String,
    status: Number,
    description: String,
    component_name: String,
    file_name: String,
});
  
const menusSchema = mongoose.Schema({
    strict: false
});

class Menu {

    async getMainMenus(id, groupPortalName) {

        let groupName = '';
        let db = global.db;

        if (groupPortalName == 'em') {

          groupName = 'employee'

        } else {

          groupName = groupPortalName;

        }

        let query = "SELECT m.*, grm.status, grm.description, grm.component_name, grm.file_name FROM group_role_menu grm INNER JOIN menu m ON grm.menus_id = m.id WHERE grm.group_role_id = (SELECT id FROM group_roles WHERE id = (SELECT group_role_id FROM user_roles WHERE user_id = ?" +
          " AND group_role_id = (SELECT gr.id FROM group_roles gr WHERE gr.link LIKE ? OR gr.group_name LIKE ? ))) ORDER BY m.parent_id, m.menu_order ASC;";
    
        return await db.runPoolQuery( query, [id, groupPortalName, groupName] );

    }
      
    
    async getPortals( id ) {
    
        var query = "SELECT r.*, gr.link, ur.status, gr.group_name FROM roles r INNER JOIN user_roles ur ON ur.role_id = r.id INNER JOIN  group_roles gr ON r.id = gr.roles_portal_id WHERE ur.user_id = ?";

        let db = global.db;
        return await db.runPoolQuery( query, [ id ] );

    }

    async getMainSubMenus( pageName ) {
    
        var query = `SELECT 
                        m.*, 
                        grm.status, 
                        grm.description, 
                        grm.component_name, 
                        grm.file_name 
                            FROM menu m 
                            INNER JOIN group_role_menu grm ON grm.menus_id = m.id WHERE m.parent_id IN ( 
                                SELECT menus_id FROM group_role_menu WHERE component_name = ? ) 
                                AND grm.group_role_id IN ( 
                                    SELECT group_role_id FROM group_role_menu WHERE component_name = ? 
                                    ) 
                        GROUP BY m.menu_title ORDER BY m.parent_id, m.menu_order ASC`;
        
        let db = global.db;

        return await db.runPoolQuery(query, [ pageName, pageName ]);

    }

}

exports.Menu = Menu;
exports.menusCacheSchema = menusCacheSchema;
exports.menus = menus;
exports.menusSchema = menusSchema;