const config = require("../../config");
const mysql = require("mysql"); 
const mongoose = require('mongoose');

class Database {

    pool = null;

    constructor() { }

    /**
     * create pool connection
     */

    createPool() {

        this.pool = mysql.createPool(config.db, { multipleStatements: config.multi_statement });

    }

    connectMongoDb() {

        mongoose.connect(config.mdb.URL, {
            useNewUrlParser: true,
            useUnifiedTopology: true
        }
        ).then(() => {
            console.log('Successfully connected');
        })
            .catch((err) => {

                console.log('Connection not found');
                console.log(err);
            process.exit("Mongo db not working");
        });

    }

    /**
     * 
     * @param {String} query query sting 
     * @param {Array} params query parameters to avoid injections
     * @returns {promise} return promise to check result
     */

    runPoolQuery(query, params = [], is_connection_retain = false) {

        return new Promise((resolve, reject) => {

            this.pool.getConnection((err, connection) => {

                if (err) {

                    reject(err)

                }
                else {

                    connection.query(query, params, (err, results, field) => {

                        if (!is_connection_retain) {
                            connection.release();
                        }

                        if (err) {

                            reject(err)

                        }
                        else {

                            let res = JSON.parse(JSON.stringify(results));

                            resolve(res);

                        }

                    })

                }

            });


        });

    }

    shutdownFunction() {
        
        if (this.pool) {

            this.pool.end((err) => {

                if (err) {

                    console.log("Error get on ending connection. Trying to shutdown all connections");

                    this.pool.getConnection((err, connection) => {

                        if (err) {

                            console.log("cannot get connections from pool");

                        }
                        else {

                            connection.destroy();

                        }

                    })

                    this.pool.end();

                }
                else {

                    console.log("connection ended successfully!");

                }

            });

        }

    }


}

module.exports = Database;