const express = require('express');
const bodyParser = require('body-parser');
const path = require('path');
const http = require('http');
const fileUpload = require('express-fileupload');
const cors = require('cors');
const helmet = require('helmet');
const xss = require('xss-clean');
const config = require('../config');
///new
const { sequelize} = require('../config');
const responseSize = require('express-response-size');
const Database = require('./configuration/database');
const api = require('./www/routes');
const { validate, ValidationError, Joi } = require('express-validation');
const swaggerUI = require('swagger-ui-express');
const swaggerJson = require('../swagger.json')
const StartupQueries = require('./configuration/system_setup_queries/StartupQueries');
const { logger } = require('handlebars');
//const cornJob = require('./api2/3.0/PayrollModule/helper/cronjobs/payroll');


const app = express(); // initialize express 
app.use(fileUpload());  //initialize file upload constructor
app.use(cors()); // cross origin enable
app.use(xss()); // xss security
app.use(helmet()); // http request header security
app.use(bodyParser.json({ limit: config.bodyParserJsonMemLimit })); // body parser limit upto 100 kb
app.use(bodyParser.urlencoded({ limit: config.bodyParserUrlEncMemLimit, extended: true, parameterLimit: config.bodyParserParamMemLimit })); // body parsing url encoding upto 100kb limit and accept 100 parameters
app.use(responseSize()); // set response size in bytes
app.use(express.static(path.join(__dirname, './../server/api2/3.0/Payroll/uploads/invoice'))); // static path

/**
 * sequelize db connection
 */

// const connectWithDB = async () => {
//   try {
//     await sequelize.authenticate();
//     logger.INFO("connection to Database Established")
//   } catch (error) {
//     logger.ERROR("there is some error")
//   }
// }
// connectWithDB();

/**
 * node js logger
 */

const opts = {
  logDirectory: path.join(__dirname, '../logs'),
  fileNamePattern: 'logs-<DATE>.log',
  dateFormat: 'YYYY.MM.DD',
};

const log = require('simple-node-logger').createRollingFileLogger(opts);

global.log = log;

const db = new Database();

global.db = db;

db.createPool();

const startupQueryConst = new StartupQueries();

startupQueryConst.databseGlobalObjectSet();

global.config = config;

app.use('/static/media', express.static(path.join(__dirname, '../uploads')));
app.use('/payroll/media', express.static(__dirname + '/api2/3.0/PayrollModule/uploads'));
app.use("/api-docs", swaggerUI.serve, swaggerUI.setup(swaggerJson));
app.use("/api2", api);

/**
 * api filed validation handler generic middleware
 */

app.use((error, req, res, next) => {

  if (error instanceof ValidationError) {

    return res.status(error.statusCode).json(error)

  }

  res.status(error.status || 500);

  res.json({

    message: error.message

  });

});

const port = config.server.PORT || '3000';

const server = http.createServer(app);

server.listen(port, async () => {
  //cornJob();
  console.log(`Running on localhost:${port}`);
});